<?php
// tm web analytics
// Copyright since 2011 by Yves Goergen
// Website: http://unclassified.de/go/tomo
// Licence: GNU General Public License (GPL) Version 3
// Encoding: UTF-8 (without BOM) (auto-detect: )
//
// This file contains the ContentType class.

// Content type data.
//
class ContentType
{
	// Path to the content-type images, MUST end with a trailing slash if non-empty.
	public static $contenttypeImgPath = '';

	// File content types array. This array is generated by the tool script
	// compile-content-types.php. It maps each file name extension to either a type code and
	// description or to another type code. The type codes are internal and not intended to match
	// similar-looking MIME content-types.
	//
	private static $contentTypes = array(
		'swf'=>array('application/adobe-flash','Adobe Flash application'),
		'class'=>array('application/java-class','Java bytecode class'),
		'msi'=>array('application/ms-installer','Microsoft Installer package'),
		'bat'=>array('application/ms-script','Microsoft script application'),
		'cmd'=>'bat',
		'vbs'=>'bat',
		'dll'=>array('application/ms-windows','Microsoft Windows application'),
		'exe'=>'dll',
		'sh'=>array('application/sh','Shell script application'),
		'7z'=>array('archive/7zip','7-Zip archive'),
		'apk'=>array('archive/android-package','Android application package'),
		'bz'=>array('archive/bzip','BZip archive'),
		'bz2'=>array('archive/bzip2','BZip2 archive'),
		'cpio'=>array('archive/cpio','CPIO archive'),
		'deb'=>array('archive/debian-package','Debian package'),
		'udeb'=>'deb',
		'gz'=>array('archive/gzip','GZip archive'),
		'jar'=>array('archive/jar','Java Archive'),
		'lzma'=>array('archive/lzma','LZMA archive'),
		'xpi'=>array('archive/mozilla-xpinstall','Mozilla add-on'),
		'cab'=>array('archive/ms-cab','Microsoft Cabinet archive'),
		'rar'=>array('archive/rar','RAR archive'),
		'tar'=>array('archive/tar','TAR archive'),
		'zip'=>array('archive/zip','ZIP archive'),
		'aac'=>array('audio/aac','AAC audio'),
		'ac3'=>array('audio/ac3','AC3 audio'),
		'aif'=>array('audio/aiff','AIFF audio'),
		'aiff'=>'aif',
		'aifc'=>'aif',
		'mid'=>array('audio/midi','MIDI audio'),
		'midi'=>'mid',
		'mpga'=>array('audio/mpeg','MPEG audio'),
		'mp2'=>'mpga',
		'mp2a'=>'mpga',
		'mp3'=>'mpga',
		'm2a'=>'mpga',
		'm3a'=>'mpga',
		'mp4a'=>array('audio/mpeg4','MPEG4 audio'),
		'wma'=>array('audio/ms-wma','Microsoft Windows Media audio'),
		'oga'=>array('audio/ogg','Ogg audio'),
		'ogg'=>'oga',
		'spx'=>'oga',
		'ram'=>array('audio/real','Real audio'),
		'ra'=>'ram',
		'wav'=>array('audio/wave','WAVE audio'),
		'cnf'=>array('document/config','Text configuration file'),
		'conf'=>'cnf',
		'config'=>'cnf',
		'inf'=>'cnf',
		'ini'=>'cnf',
		'csv'=>array('document/csv','Comma-separated values'),
		'diff'=>array('document/diff','Unified diff file'),
		'patch'=>'diff',
		'eml'=>array('document/email','E-mail document'),
		'mime'=>'eml',
		'mdb'=>array('document/ms-access','Microsoft Access database'),
		'xls'=>array('document/ms-excel','Microsoft Excel document'),
		'xlsx'=>'xls',
		'chm'=>array('document/ms-htmlhelp','Microsoft HTML help'),
		'ppt'=>array('document/ms-powerpoint','Microsoft PowerPoint presentation'),
		'pptx'=>'ppt',
		'pps'=>'ppt',
		'ppsx'=>'ppt',
		'vsd'=>array('document/ms-visio','Microsoft Visio document'),
		'doc'=>array('document/ms-word','Microsoft Word document'),
		'docx'=>'doc',
		'dot'=>array('document/ms-word-template','Microsoft Word document template'),
		'dotx'=>'dot',
		'wri'=>array('document/ms-write','Microsoft Write document'),
		'xps'=>array('document/ms-xps','Microsoft XPS document'),
		'odc'=>array('document/oasis-chart','OpenDocument chart'),
		'otc'=>array('document/oasis-chart-template','OpenDocument chart template'),
		'odb'=>array('document/oasis-database','OpenDocument database'),
		'odf'=>array('document/oasis-formula','OpenDocument formula'),
		'odft'=>array('document/oasis-formula-template','OpenDocument formula template'),
		'odg'=>array('document/oasis-graphics','OpenDocument graphics'),
		'odp'=>array('document/oasis-presentation','OpenDocument presentation'),
		'otp'=>array('document/oasis-presentation-template','OpenDocument presentation template'),
		'ods'=>array('document/oasis-spreadsheet','OpenDocument spreadsheet'),
		'ots'=>array('document/oasis-spreadsheet-template','OpenDocument spreadsheet template'),
		'odt'=>array('document/oasis-text','OpenDocument text'),
		'otm'=>array('document/oasis-text-master','OpenDocument text master'),
		'ott'=>array('document/oasis-text-template','OpenDocument text template'),
		'pdf'=>array('document/pdf','Adobe PDF document'),
		'txt'=>array('document/plaintext','Plain text'),
		'text'=>'txt',
		'eps'=>array('document/postscript','PostScript document'),
		'ps'=>'eps',
		'rtf'=>array('document/rtf','Rich Text document'),
		'sxc'=>array('document/staroffice-calc','Sun StarOffice Calc document'),
		'sxd'=>array('document/staroffice-draw','Sun StarOffice Draw document'),
		'sxi'=>array('document/staroffice-impress','Sun StarOffice Impress presentation'),
		'sxw'=>array('document/staroffice-writer','Sun StarOffice Writer document'),
		'tex'=>array('document/tex','TeX document'),
		'vcs'=>array('document/vcalendar','vCalendar file'),
		'vcf'=>array('document/vcard','vCard file'),
		'wpd'=>array('document/wordperfect','WordPerfect document'),
		'crt'=>array('document/x509-cert','X.509 certificate'),
		'der'=>'crt',
		'xml'=>array('document/xml','XML document'),
		'eot'=>array('font/ms-eot','Microsoft web font'),
		'otf'=>array('font/opentype','OpenType font'),
		'ttf'=>array('font/truetype','TrueType font'),
		'pfa'=>array('font/type1','Type1 font'),
		'pfb'=>'pfa',
		'pfm'=>'pfa',
		'kml'=>array('generic/google-earth-kml','Google Earth KML'),
		'kmz'=>array('generic/google-earth-kmz','Google Earth KMZ'),
		'pgp'=>array('generic/pgp-encrypted','PGP encrypted file'),
		'asc'=>array('generic/pgp-signature','PGP signature'),
		'sig'=>'asc',
		'dng'=>array('image/adobe-dng','Adobe digital negative'),
		'psd'=>array('image/adobe-photoshop','Adobe Photoshop image'),
		'bmp'=>array('image/bitmap','Bitmap image'),
		'gif'=>array('image/gif','GIF image'),
		'jpeg'=>array('image/jpeg','JPEG image'),
		'jpg'=>'jpeg',
		'jpe'=>'jpeg',
		'jp2'=>array('image/jpeg2000','JPEG2000 image'),
		'ico'=>array('image/ms-icon','Microsoft icon image'),
		'wmf'=>array('image/ms-wmf','Microsoft Windows Metafile graphic'),
		'png'=>array('image/png','PNG image'),
		'svg'=>array('image/svg','SVG graphic'),
		'svgz'=>'svg',
		'tiff'=>array('image/tiff','TIFF image'),
		'tif'=>'tiff',
		'xbm'=>array('image/xbitmap','X bitmap image'),
		'xpm'=>array('image/xpixmap','X pixmap image'),
		's'=>array('source/asm','Assembler source code'),
		'asm'=>'s',
		'bas'=>array('source/basic','BASIC source code'),
		'c'=>array('source/c','C/C++ source code'),
		'h'=>array('source/c-header','C/C++ header code'),
		'cc'=>array('source/cpp','C/C++ source code'),
		'cxx'=>'cc',
		'cpp'=>'cc',
		'hh'=>array('source/cpp-header','C/C++ header code'),
		'hpp'=>'hh',
		'cs'=>array('source/csharp','C# source code'),
		'css'=>array('source/css','Cascading stylesheet'),
		'html'=>array('source/html','HTML source code'),
		'htm'=>'html',
		'java'=>array('source/java','Java source code'),
		'js'=>array('source/javascript','JavaScript source code'),
		'vb'=>array('source/visual-basic','Visual Basic source code'),
		'p'=>array('source/pascal','Pascal source code'),
		'pas'=>'p',
		'pl'=>array('source/perl','Perl source code'),
		'php'=>array('source/php','PHP source code'),
		'py'=>array('source/python','Python source code'),
		'rb'=>array('source/ruby','Ruby source code'),
		'rbw'=>'rb',
		'wsdl'=>array('source/wsdl','WSDL source code'),
		'xaml'=>array('source/xaml','XAML source code'),
		'xhtml'=>array('source/xhtml','XHTML web page'),
		'dtd'=>array('source/xml-dtd','XML document type definition'),
		'xsd'=>array('source/xsd','XML schema'),
		'xsl'=>array('source/xsl','XML stylesheet'),
		'xslt'=>array('source/xslt','XSL transformation file'),
		'xul'=>array('source/xul','Mozilla XUL source code'),
		'3gp'=>array('video/3gpp','3GPP video'),
		'qt'=>array('video/apple-quicktime','Apple QuickTime video'),
		'mov'=>'qt',
		'h264'=>array('video/h264','H.264 video'),
		'mj2'=>array('video/mjp2','MJP2 video'),
		'mjp2'=>'mj2',
		'mpeg'=>array('video/mpeg','MPEG video'),
		'mpg'=>'mpeg',
		'mpe'=>'mpeg',
		'm1v'=>'mpeg',
		'm2v'=>'mpeg',
		'mp4'=>array('video/mpeg4','MPEG4 video'),
		'mp4v'=>'mp4',
		'mpg4'=>'mp4',
		'asf'=>array('video/ms-asf','Microsoft ASF video'),
		'asx'=>'asf',
		'avi'=>array('video/ms-avi','Microsoft AVI video'),
		'vc1'=>array('video/ms-vc1','Microsoft VC-1 video'),
		'wm'=>array('video/ms-wmv','Microsoft Windows Media video'),
		'wmv'=>'wm',
		'ogv'=>array('video/ogg','Ogg video'),
		);

	// Available icons for ContentTypeIconSprite()
	private static $icons = array(
		'application',
		'application_java-class',
		'application_ms-installer',
		'application_ms-script',
		'application_ms-windows',
		'application_sh',
		'archive',
		'archive_android-package',
		'archive_debian-package',
		'archive_mozilla-xpinstall',
		'audio',
		'audio_midi',
		'document',
		'document_config',
		'document_csv',
		'document_diff',
		'document_email',
		'document_ms-access',
		'document_ms-excel',
		'document_ms-htmlhelp',
		'document_ms-powerpoint',
		'document_ms-visio',
		'document_ms-word',
		'document_ms-write',
		'document_ms-xps',
		'document_oasis-chart-template',
		'document_oasis-chart',
		'document_oasis-database',
		'document_oasis-formula-template',
		'document_oasis-formula',
		'document_oasis-graphics',
		'document_oasis-presentation-template',
		'document_oasis-presentation',
		'document_oasis-spreadsheet-template',
		'document_oasis-spreadsheet',
		'document_oasis-text-master',
		'document_oasis-text-template',
		'document_oasis-text',
		'document_pdf',
		'document_plaintext',
		'document_postscript',
		'document_rtf',
		'document_staroffice-calc',
		'document_staroffice-draw',
		'document_tex',
		'document_vcalendar',
		'document_vcard',
		'document_x509-cert',
		'document_xml',
		'font_opentype',
		'font_truetype',
		'font_type1',
		'generic_google-earth-kml',
		'generic_google-earth-kmz',
		'generic_pgp-encrypted',
		'generic_pgp-signature',
		'image',
		'image_adobe-dng',
		'image_adobe-photoshop',
		'image_ms-wmf',
		'image_svg',
		'source',
		'source_c-header',
		'source_c',
		'source_cpp-header',
		'source_cpp',
		'source_csharp',
		'source_html',
		'source_java',
		'source_javascript',
		'source_pascal',
		'source_perl',
		'source_php',
		'source_python',
		'source_ruby',
		'source_xhtml',
		'source_xml-dtd',
		'source_xsd',
		'source_xslt',
		'unknown',
		'video',
		);

	// Gets the internal content type for the specified file name.
	//
	// The content type is defined in an internal array and matched by the file name extension.
	//
	// filename = (string) File name to get the type of
	//
	// Returns (string) Type code, or {{null}} for unknown file types.
	//
	public static function GetContentTypeForFileName($filename)
	{
		if (!is_string($filename))
			throw new InvalidArgumentException('Invalid argument type: $filename');

		if (preg_match('~[^.].([^.]+)$~', $filename, $m))
		{
			$ext = strtolower($m[1]);

			$type = self::$contentTypes[$ext];
			if (is_array($type))
			{
				return $type[0];
			}
			else if (is_string($type))
			{
				// Resolve redirection to another file name extension
				$type = self::$contentTypes[$type];
				if (is_array($type))
				{
					return $type[0];
				}
				// Corrupt types array, pointing to an invalid other extension
			}
			// Unknown type
			return null;
		}
	}

	// Gets the description of the specified content type.
	//
	// contentType = (string) Content type to get the description of
	//
	// Returns (string) Content type description.
	//
	public static function GetContentTypeDescription($contentType)
	{
		if (isset($contentType))
		{
			if (!is_string($contentType))
				throw new InvalidArgumentException('Invalid argument type: $contentType');
			if (!strlen($contentType))
				$contentType = null;
		}

		foreach (self::$contentTypes as $type)
		{
			if (is_array($type))
			{
				if ($type[0] == $contentType) return $type[1];
			}
		}
		return null;
	}

	// Gets the file name for a file content type icon, including the configured path.
	//
	// contentType = (string) File content type
	//
	public static function ContentTypeIcon($contentType)
	{
		$path = self::$contenttypeImgPath;
		$fileName = str_replace('/', '_', $contentType);
		if (!file_exists($path . $fileName . '.png'))
			$fileName = preg_replace('~/.*$~', '', $contentType);
		if (!file_exists($path . $fileName . '.png'))
			$fileName = 'unknown';

		return $path . $fileName . '.png';
	}
	
	public static function ContentTypeIconSprite($contentType)
	{
		$fileName = str_replace('/', '_', $contentType);
		if (!in_array($fileName, self::$icons))
			$fileName = preg_replace('~/.*$~', '', $contentType);
		if (!in_array($fileName, self::$icons))
			$fileName = 'unknown';
		return $fileName;
	}

} // class ContentType
?>
